/*
 * Copyright (c) 2004 Hamish Rodda <rodda@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef ATTRIBUTES_H
#define ATTRIBUTES_H

#include <qobject.h>
#include <qvaluelist.h>
#include <qcombobox.h>

#include <X11/extensions/NVCtrl.h>

class SettingsBridge;
class KConfig;

// FIXME store changed settings per-screen and displaymask...
class AbstractAttribute : public QObject
{
	Q_OBJECT

public:
	AbstractAttribute(SettingsBridge* bridge, QWidget* widget);
	
	bool isValid() const;
	void setValid(bool valid);
	
	SettingsBridge* bridge() const;
	virtual QWidget* widget() const;
	
	virtual bool isChanged() const = 0;
	virtual void commitChange() = 0;
	virtual void revertChange() = 0;
	
	virtual void makeConnections() = 0;
	virtual void update() = 0;
	virtual void checkUpdate(int attribute, int displayMask) = 0;
	
	virtual bool concernsAttribute(int a, int dm = 0) const = 0;
	virtual void setAttribute(int screen, int a, int v) = 0;
	virtual void save(KConfig* config, int screen) const = 0;
	
signals:
	void changed(bool c);
	
private:
	SettingsBridge* m_bridge;
	QWidget* m_widget;
	bool m_valid;
};

class SimpleAttribute : public AbstractAttribute
{
	Q_OBJECT

public:
	SimpleAttribute(SettingsBridge* b, QWidget* widget, int attribute, int displayMask = 0, bool reverse = false);
	
	int attribute() const;
	int displayMask() const;
	int value(int screen = -1) const;
	
	virtual bool isChanged() const;
	int proposedValue(int screen = -1) const;
	virtual void commitChange();
	virtual void revertChange();
	
	bool appliesImmediately() const;
	void setAppliesImmediately(bool immediate = true);
	
	virtual void makeConnections();
	virtual void update();
	virtual void checkUpdate(int attribute, int displayMask);
	
	virtual bool concernsAttribute(int a, int dm) const;
	virtual void setAttribute(int screen, int a, int v);
	virtual void save(KConfig* config, int screen) const;

	_NVCTRLAttributeValidValues validValues;

public slots:
	void setAttribute(int v);
	void setAttribute(bool v);
	
private:
	void checkChanged();

	uint m_attribute, m_displayMask;
	/// Reversed bool values is _only_ for the GUI representation
	bool	m_reverse : 1,
			m_changed : 1,
			m_appliesImmediately : 1;
	QMap<int,int> m_proposedValues;
};

/// Special class for combining On/Off bool attribute with Integer attribute when option1 == On
class CombinedAttribute : public AbstractAttribute
{
	Q_OBJECT
	
public:
	CombinedAttribute(SettingsBridge* bridge, QWidget* widget, int boolAttribute, int intAttribute);
	
	virtual QComboBox* widget() const;
	
	virtual void makeConnections();
	virtual void update();
	virtual void checkUpdate(int attribute, int displayMask);

	virtual bool isChanged() const;
	virtual void commitChange();
	virtual void revertChange();
	
	virtual bool concernsAttribute(int a, int dm) const;
	virtual void setAttribute(int screen, int a, int v);
	virtual void save(KConfig* config, int screen) const;

public slots:
	void setAttribute(int value);
	
private:
	SimpleAttribute* m_boolAttrib;
	SimpleAttribute* m_intAttrib;
};

#endif
