/*
 * Copyright (c) 2004 Hamish Rodda <rodda@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "nvidiamodule.h"

#include <qcheckbox.h>
#include <qwhatsthis.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qdesktopwidget.h>

#include <kdebug.h>
#include <klocale.h>
#include <kcmodule.h>
#include <kglobal.h>
#include <kgenericfactory.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <kapplication.h>

#include "settingsbridge.h"

#include <X11/Xlib.h>
extern "C" {
	#include <X11/extensions/NVCtrlLib.h>
}

NvidiaModule::NvidiaModule(QWidget *parent, const char *name, const QStringList&)
    : KCModule(parent, name)
	, m_settings(new SettingsBridge())
{
	if (!m_settings->isValid()) {
		(new QVBoxLayout(this))->add(new QLabel(i18n("<qt>Your X server does not support the NVIDIA-CONTROL extension, or there are no screens supported by the extension.</qt>"), this));
		return;
	}
	
	(new QVBoxLayout(this))->setAutoAdd(true);
	
	setButtons(KCModule::Apply | KCModule::Default);
	
	connect(m_settings, SIGNAL(changed(bool)), SIGNAL(changed(bool)));
	
	kapp->installX11EventFilter(this);
}

bool NvidiaModule::x11Event( XEvent * e )
{
	if (e->type != m_settings->eventBase() + ATTRIBUTE_CHANGED_EVENT)
		return false;
	
	XNVCtrlAttributeChangedEvent* nve = reinterpret_cast<XNVCtrlAttributeChangedEvent*>(e);
	kdDebug() << "NV-CONTROL attribute changed event: screen " << nve->screen << ", display_mask " << nve->display_mask << ", attribute " << nve->attribute << ", value " << nve->value << endl;
	m_settings->attributeChanged(nve->attribute, nve->display_mask, nve->screen);
	return true;
}

void NvidiaModule::defaults()
{
	// FIXME load hardware defaults
	m_settings->revertChanges();
}

void NvidiaModule::load()
{
	m_settings->loadFromDisk();
}

void NvidiaModule::save()
{
	m_settings->commitChanges();
	m_settings->saveToDisk();
}

NvidiaModule::~ NvidiaModule( )
{
	delete m_settings;
}

#include "nvidiamodule.moc"
