/*
 * Copyright (c) 2004 Hamish Rodda <rodda@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef SETTINGSBRIDGE_H
#define SETTINGSBRIDGE_H

#include <qobject.h>
#include <qvaluelist.h>
#include <qintdict.h>
#include <qcombobox.h>

class SettingsBridge;
class AbstractAttribute;

/**
 * Class which sets up and manipulates bindings between widgets and NVIDIA-CONTROL attributes.
 */
class SettingsBridge : public QObject
{
	Q_OBJECT

public:
	SettingsBridge(bool loadOnly = false, QObject *parent = 0, const char *name = 0);
	
	bool isValid() const;
	bool screenValid(int screen) const;
	const QValueList<int>& validScreens() const;
	int eventBase() const;
	
	int displayMaskForName(const QString& displayName, int screen = -1) const;
	QString displayNameForMask(int displayMask, int screen = -1) const;
	
	void registerWidget(QWidget* widget, uint attribute, bool reverse = false, QWidget* containerWidget = 0L);
	void registerWidget(QWidget* widget, uint attribute, int displayMask, bool appliesImmediately = false, QWidget* containerWidget = 0L);
	void registerAttribute(AbstractAttribute* a, QWidget * containerWidget = 0L);
	void deregisterWidget(QWidget* widget);
	
	void commitChanges();
	// UNUSED - not needed by kcontrol
	void revertChanges();
	
	void loadFromDisk(bool registeredOnly = true);
	void saveToDisk();
	
	int currentScreen() const;
	
	int queryAttribute(int attribute, int displayMask = 0, int screen = -1) const;
	QString queryString(int attribute, int displayMask = 0, int screen = -1) const;
	void setAttribute(int attribute, int value, int displayMask = 0, int screen = -1);
	
	void attributeChanged(int screen, uint displayMask, uint attribute);
	
	AbstractAttribute* attributeObject(int attribute, int displayMask = 0) const;
	QString attributeName(int attribute, int displayMask = 0, int screen = 0) const;

signals:
	void changed(bool changed);

public slots:
	void setCurrentScreen(int screen);

private slots:
	void slotAttributeChanged(bool c);

private:
	bool	m_loadOnly : 1,
			m_bridgeValid : 1,
			m_changed : 1;
	int		m_currentScreen,
			m_eventBase;
	
	QMap<const QWidget*, AbstractAttribute*>	m_widgets;
	QValueList<AbstractAttribute*>				m_attributes;
	QValueList<int>								m_validScreens;
	QMap<int, QMap<QString, int> >				m_displays;
};

#endif
